'use strict';

/**
 * Gives same value.
 * @param v a value
 * @returns v
 */
function id(v) {
    return v;
}

/**
 * Compares two values.
 * @param a a value
 * @param b another value
 * @returns a<b: -1, a=b: 0, a>b: 1
 */
function cmp(a, b) {
    return a < b ? -1 : (a > b ? 1 : 0);
}

/**
 * Gets unique set of values.
 * @param x an iterable
 * @param fm map function (v, i, x)
 */
function uniqueSet(x, fm = null) {
    if (!fm)
        return new Set(x);
    var s = new Set(), i = -1;
    for (var v of x)
        s.add(fm(v, ++i, x));
    return s;
}

function unionMap$(x, y, fm = null) {
    var s = uniqueSet(x, fm);
    var fm = fm || id, i = -1;
    for (var v of y) {
        var v1 = fm(v, ++i, y);
        if (!s.has(v1))
            x.push(v);
    }
    return x;
}
function unionDual$(x, y, fc = null, fm = null) {
    var fc = fc || cmp, fm = fm || id;
    var x1 = x.map(fm), j = -1;
    y: for (var v of y) {
        var v1 = fm(v, ++j, y);
        for (var u1 of x1)
            if (fc(u1, v1) === 0)
                continue y;
        x.push(v);
    }
    return x;
}
/**
 * Gives values present in any array.
 * @param x an array (updated)
 * @param y another array
 * @param fc compare function (a, b)
 * @param fm map function (v, i, x)
 * @returns x
 */
function union$(x, y, fc = null, fm = null) {
    if (fc)
        return unionDual$(x, y, fc, fm);
    else
        return unionMap$(x, y, fm);
}

module.exports = union$;
